#ifndef DEFAULT_H
#define DEFAULT_H

/* default width of a column */
#define DEF_COLUMNWIDTH       12

/* default precision of a printed value */
#define DEF_PRECISION          6

/* real maximum precision of an edited/examined value */
#define DEF_REALPRECISION     12

/* default is no scientific notation for numbers */
#define DEF_SCIENTIFIC         0

/* default is left adjusted cell */
#define DEF_ADJUST          LEFT

/* character attribute for cell and row numbers */
#define DEF_NUMBER        A_BOLD

/* character attribute for cell cursor */
#define DEF_CELLCURSOR A_REVERSE

/* character attribute for selected menu choice */
#define DEF_MENU       A_REVERSE

/* maximum number of sort keys */
#define MAX_SORTKEYS           8

#endif
/*{{{}}}*/
