#ifndef MISC_H
#define MISC_H

void order(int *x, int *y);
long int posnumber(const char *s, const char **endptr);
char *strmalloc(const char *str);
const char *isfinite(double x);

#endif
/*{{{}}}*/
