#ifndef CAT_H
#define CAT_H

#define catgets(catd,setid,msgid,msg) msg

#define OHWELL     catgets(catd,1,1,"ERROR")
#define DIV0       catgets(catd,1,2,"division by 0")
#define DIVTYPE    catgets(catd,1,3,"wrong types for / operator")
#define MULTYPE    catgets(catd,1,4,"wrong types for * operator")
#define ADDTYPE    catgets(catd,1,5,"wrong types for + operator")
#define SUBTYPE    catgets(catd,1,6,"wrong types for - operator")
#define NEGTYPE    catgets(catd,1,7,"wrong type for - operator")
#define POWTYPE    catgets(catd,1,8,"wrong types for ^ operator")
#define MISSOPR    catgets(catd,1,9,"missing operator")
#define TOOMANY    catgets(catd,1,10,"too many arguments")
#define CPEXPCT    catgets(catd,1,11,") expected")
#define VLEXPCT    catgets(catd,1,12,"value expected")
#define PARSERR    catgets(catd,1,13,"parse error after term")
#define RELTYPE    catgets(catd,1,14,"type mismatch for relational operator")
#define NOFUNC     catgets(catd,1,15,"No such function")
#define NOLABEL    catgets(catd,1,16,"No such label")
#define NEGLOC     catgets(catd,1,17,"Negative index")
#define POW00      catgets(catd,1,18,"0^0 is not defined")
#define POWDOM     catgets(catd,1,19,"^ caused a domain error")
#define ISLOCKED   catgets(catd,1,20,"Cell is locked")
#define NOTAG      catgets(catd,1,21,"Unknown tag %c in line %d")
#define SHADOWBOO2 catgets(catd,1,22,"Trying to shadow cell (%d,%d,%d) in line %d")
#define XERR       catgets(catd,1,23,"Parse error for x position in line %d")
#define YERR       catgets(catd,1,24,"Parse error for y position in line %d")
#define ZERR       catgets(catd,1,25,"Parse error for z position in line %d")
#define OPTERR     catgets(catd,1,26,"Invalid option %c in line %d")
#define EXPRERR    catgets(catd,1,27,"Expression syntax error in line %d")
#define WIDTHERR   catgets(catd,1,28,"Parse error for width in line %d")
#define PRECERR    catgets(catd,1,29,"Parse error for precision in line %d")
#define ADJERR     catgets(catd,1,30,"Parse error for adjustment in line %d")
#define NOTFINITE  catgets(catd,1,31,"Not a (finite) floating point number")
#define UFLOW      catgets(catd,1,32,"Floating point range underflow")
#define OFLOW      catgets(catd,1,33,"Floating point range overflow")
#define UNCOMPAR   catgets(catd,1,34,"Warning, uncomparable elements")
#define NOREGION   catgets(catd,1,35,"No region selected")
#define YES        catgets(catd,1,36,"YY)es")
#define NO         catgets(catd,1,37,"NN)o")
#define ADLEFT     catgets(catd,1,38,"LL)eft")
#define ADRIGHT    catgets(catd,1,39,"RR)ight")
#define ADCENTERED catgets(catd,1,40,"CC)entered")
#define PRECISION  catgets(catd,1,41,"PP)recision")
#define DOLOCK     catgets(catd,1,42,"OlO)ck")
#define DOSCI      catgets(catd,1,43,"00).123e1")
#define NOSCI      catgets(catd,1,44,"11).23")
#define LABEL      catgets(catd,1,45,"AlA)bel")
#define ASXDR      catgets(catd,1,46,"XX)DR")
#define ASASCII    catgets(catd,1,47,"AA)SCII")
#define TBL        catgets(catd,1,48,"TT)bl")
#define LATEX      catgets(catd,1,49,"LL)aTeX")
#define SAVETEXT   catgets(catd,1,50,"FF)ormatted text")
#define LOAD       catgets(catd,1,51,"LL)oad")
#define SAVE       catgets(catd,1,52,"SS)ave")
#define QUIT       catgets(catd,1,53,"QQ)uit")
#define ATTR       catgets(catd,1,54,"AA)ttribute")
#define SHADOW     catgets(catd,1,55,"SS)hadow")
#define INX        catgets(catd,1,56,"CC)olumn")
#define INY        catgets(catd,1,57,"RR)ow")
#define INZ        catgets(catd,1,58,"DD)epth")
#define INSERT     catgets(catd,1,59,"II)nsert")
#define CLEAR      catgets(catd,1,60,"EclE)ar")
#define MOVE       catgets(catd,1,61,"MM)ove")
#define COPY       catgets(catd,1,62,"CC)opy")
#define TOMENU     catgets(catd,1,63,"Use / for menu")
#define ABOUT      catgets(catd,1,64,"VV)ersion")
#define CWIDTH     catgets(catd,1,65,"WW)idth")
#define DELETE     catgets(catd,1,66,"DD)elete")
#define SORT       catgets(catd,1,67,"OsO)rt")
#define ADDKEY     catgets(catd,1,68,"Aa)dd key")
#define ASCEND     catgets(catd,1,69,"Aa)scending")
#define DESCEND    catgets(catd,1,70,"Dd)escending")
#define SORTIT     catgets(catd,1,71,"SS)ort region")
#define LOADXSHEET catgets(catd,1,72,"Load sheet from XDR file:")
#define LOADPSHEET catgets(catd,1,73,"Load sheet from ASCII file:")
#define SAVEXSHEET catgets(catd,1,74,"Save sheet to XDR file:")
#define SAVEPSHEET catgets(catd,1,75,"Save sheet to ASCII file:")
#define SAVEMENU   catgets(catd,1,76,"Save sheet file format:")
#define LOADMENU   catgets(catd,1,77,"Load sheet file format:")
#define LEAVE      catgets(catd,1,78,"Sheet modified, leave anyway?")
#define MAINMENU   catgets(catd,1,79,"Main menu:")
#define COLWIDTH   catgets(catd,1,80,"Column width:")
#define CELLATTR   catgets(catd,1,81,"Cell attribute:")
#define SHADOWCELL catgets(catd,1,82,"Shadow cell:")
#define CONTENTS   catgets(catd,1,83,"Cell contents:")
#define SHADOWBOO  catgets(catd,1,84,"You can not shadow cells in column 0")
#define PRECCELL   catgets(catd,1,85,"Precision for cell:")
#define PRECBLOCK  catgets(catd,1,86,"Precision for block:")
#define PAGES      catgets(catd,1,87,"List of pages to print:")
#define SAVETEXTF  catgets(catd,1,88,"Save in plain text format to file:")
#define SAVETBLF   catgets(catd,1,89,"Save in tbl format to file:")
#define SAVELATEXF catgets(catd,1,90,"Save in LaTeX format to file:")
#define BLOCKATTR  catgets(catd,1,91,"Block attribute:")
#define CLEARBLOCK catgets(catd,1,92,"Clear block:")
#define CLEARCELL  catgets(catd,1,93,"Clear cell:")
#define SHADOWCUBE catgets(catd,1,94,"Shadow block:")
#define INSCELL    catgets(catd,1,95,"Insert cell:")
#define INSCUBE    catgets(catd,1,96,"Insert block:")
#define DELCELL    catgets(catd,1,97,"Delete cell:")
#define DELBLOCK   catgets(catd,1,98,"Delete block:")
#define EDIT       catgets(catd,1,99,"Edit cell:")
#define LOCKCELL   catgets(catd,1,100,"Lock cell:")
#define LOCKBLOCK  catgets(catd,1,101,"Lock block:")
#define DOLABEL    catgets(catd,1,102,"Cell label:")
#define MOVEBLOCK  catgets(catd,1,103,"Move block:")
#define NOMARK     catgets(catd,1,104,"No block marked")
#define COPYBLOCK  catgets(catd,1,105,"Copy block:")
#define SORTBLOCK  catgets(catd,1,106,"Sort block:")
#define XPOS       catgets(catd,1,107,"X position of key vector:")
#define YPOS       catgets(catd,1,108,"Y position of key vector:")
#define ZPOS       catgets(catd,1,109,"Z position of key vector:")
#define EEKERROR   catgets(catd,1,110,"Usage: error(string message)")
#define EEKPTR     catgets(catd,1,111,"Usage: &(integer x,integer y,integer z)")
#define EEKAT      catgets(catd,1,112,"Usage: @(integer x,integer y[,integer z]) or @(location)")
#define HEREXTYPE  catgets(catd,1,113,"Usage: here.x([location])")
#define HEREYTYPE  catgets(catd,1,114,"Usage: here.y([location])")
#define HEREZTYPE  catgets(catd,1,115,"Usage: here.z([location])")
#define CTOSTYPE   catgets(catd,1,116,"Usage: ctos(integer x,integer y[,integer z]) or ctos(location)")
#define SUMTYPE    catgets(catd,1,117,"Usage: sum(location,location)")
#define NTYPE      catgets(catd,1,118,"Usage: n(location,location)")
#define INTTYPE    catgets(catd,1,119,"Usage: int(float)")
#define FRACTYPE   catgets(catd,1,120,"Usage: frac(float)")
#define LENTYPE    catgets(catd,1,121,"Usage: len(string)")
#define MINTYPE    catgets(catd,1,122,"Usage: min(location,location)")
#define ABSTYPE    catgets(catd,1,123,"Usage: abs(float|integer)")

#endif
/*{{{}}}*/
