#ifndef XDR_H
#define XDR_H

#include <sys/types.h>
/* I only want <rpc/xdr.h> but the Solaris include files would */
/* make that too much of a pain. */
#include <rpc/rpc.h>

#include "sheet.h"

bool_t xdr_cell(XDR *xdrs, Cell *cell);
bool_t xdr_column(XDR *xdrs, int *x, int *z, int *width);

#endif
/*{{{}}}*/
